import * as i0 from '@angular/core';
import { Injectable, Optional, Inject, APP_INITIALIZER, NgModule, SkipSelf } from '@angular/core';
import { DOCUMENT } from '@angular/common';

class ThemingOptions {
}

class Theming {
    constructor(_options, _rendererFactory, _document) {
        this._options = _options;
        this._rendererFactory = _rendererFactory;
        this._document = _document;
        // Readonly data.
        this._attribute = 'data-theme';
        this._renderer = this._rendererFactory.createRenderer(null, null);
        // State.
        this._currentTheme = null;
        this.setCssVariables();
        this.setFirstTheme();
    }
    get all() {
        var _a, _b;
        return (_b = (_a = this._options) === null || _a === void 0 ? void 0 : _a.themes) !== null && _b !== void 0 ? _b : [];
    }
    get current() {
        var _a;
        return (_a = this._currentTheme) !== null && _a !== void 0 ? _a : null;
    }
    set(name) {
        var _a;
        const theme = (_a = this.all) === null || _a === void 0 ? void 0 : _a.find(theme => (theme === null || theme === void 0 ? void 0 : theme.name) === name);
        if (!theme) {
            throw new Error('Invalid theme.');
        }
        const document = this._document.documentElement;
        if (!document) {
            return;
        }
        this._renderer.setAttribute(document, this._attribute, name);
        this._currentTheme = theme;
    }
    setCssVariables() {
        var _a, _b, _c;
        const style = this._renderer.createElement('style');
        this._renderer.setAttribute(style, 'id', 'theming-style');
        const variables = this.getCssVariables();
        this._renderer.setProperty(style, 'innerHTML', variables);
        const head = (_b = (_a = this._document) === null || _a === void 0 ? void 0 : _a.head) !== null && _b !== void 0 ? _b : null;
        if (!head) {
            return;
        }
        this._renderer.appendChild((_c = this._document) === null || _c === void 0 ? void 0 : _c.head, style);
    }
    getCssVariables() {
        return this.all
            .map(theme => `:root[${this._attribute}="${theme === null || theme === void 0 ? void 0 : theme.name}"] {${this.getThemeContents(theme)}}`)
            .join('\n');
    }
    getThemeContents(theme) {
        var _a;
        const themeData = (_a = theme === null || theme === void 0 ? void 0 : theme.data) !== null && _a !== void 0 ? _a : {};
        return Object.keys(themeData)
            .map(key => `${key}: ${themeData === null || themeData === void 0 ? void 0 : themeData[key]};`)
            .join(' ');
    }
    setFirstTheme() {
        var _a, _b, _c;
        const theme = (_b = (_a = this.all) === null || _a === void 0 ? void 0 : _a[0]) !== null && _b !== void 0 ? _b : null;
        if (!theme) {
            throw new Error('No theme found.');
        }
        const themeName = (_c = theme === null || theme === void 0 ? void 0 : theme.name) !== null && _c !== void 0 ? _c : null;
        return this.set(themeName);
    }
}
Theming.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.0.4", ngImport: i0, type: Theming, deps: [{ token: ThemingOptions }, { token: i0.RendererFactory2 }, { token: DOCUMENT, optional: true }], target: i0.ɵɵFactoryTarget.Injectable });
Theming.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "14.0.4", ngImport: i0, type: Theming });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.0.4", ngImport: i0, type: Theming, decorators: [{
            type: Injectable
        }], ctorParameters: function () {
        return [{ type: ThemingOptions }, { type: i0.RendererFactory2 }, { type: Document, decorators: [{
                        type: Optional
                    }, {
                        type: Inject,
                        args: [DOCUMENT]
                    }] }];
    } });

class ThemingModule {
    constructor(_parent) {
        this._parent = _parent;
        if (this._parent) {
            throw new Error('ThemingModule is already loaded; Import it in the core module only.');
        }
    }
    static forRoot(options) {
        return {
            ngModule: ThemingModule,
            providers: [
                Theming,
                { provide: APP_INITIALIZER, deps: [Theming], useFactory: initializeTheme, multi: true },
                { provide: ThemingOptions, useValue: options }
            ]
        };
    }
}
ThemingModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.0.4", ngImport: i0, type: ThemingModule, deps: [{ token: ThemingModule, optional: true, skipSelf: true }], target: i0.ɵɵFactoryTarget.NgModule });
ThemingModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "14.0.4", ngImport: i0, type: ThemingModule });
ThemingModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "14.0.4", ngImport: i0, type: ThemingModule });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.0.4", ngImport: i0, type: ThemingModule, decorators: [{
            type: NgModule,
            args: [{}]
        }], ctorParameters: function () {
        return [{ type: ThemingModule, decorators: [{
                        type: Optional
                    }, {
                        type: SkipSelf
                    }] }];
    } });
function initializeTheme(_) {
    // Note: We just need to initialize the theme.
    return () => ({});
}

class Theme {
    constructor(_name, _data) {
        this._name = _name;
        this._data = _data;
        this._parsed = this.parseData(_data !== null && _data !== void 0 ? _data : {});
    }
    get name() {
        return this._name;
    }
    get data() {
        return this._parsed;
    }
    update(data) {
        this._parsed = Object.assign(Object.assign({}, this._parsed), this.parseData(data));
        return this;
    }
    parseData(data, scope = null) {
        return Object.keys(data).reduce((previous, current) => {
            const value = data === null || data === void 0 ? void 0 : data[current];
            const token = this.getFormattedToken(current, scope);
            if (typeof value === 'object' && value !== null) {
                const formatted = this.parseData(value, token);
                return Object.assign(Object.assign({}, previous), formatted);
            }
            return Object.assign(Object.assign({}, previous), { [`--${token}`]: value });
        }, {});
    }
    getFormattedToken(key, scope = null) {
        const result = key.replace(/([A-Z])/g, ' $1');
        const current = result.split(' ').join('-').toLowerCase();
        return [scope, current].filter(item => !!item).join('-');
    }
}

/**
 * Generated bundle index. Do not edit.
 */

export { Theme, Theming, ThemingModule, ThemingOptions, initializeTheme };
//# sourceMappingURL=bravobit-bb-foundation-theming.mjs.map
