import * as i0 from '@angular/core';
import { Directive, Optional, Self, Input, HostListener, Injectable, NgModule } from '@angular/core';
import _Inputmask from 'inputmask';
import * as i1 from '@angular/cdk/platform';
import * as i2 from '@angular/forms';
import { formatDate, getCurrencySymbol, getLocaleNumberSymbol, NumberSymbol, getNumberOfCurrencyDigits } from '@angular/common';
import * as i1$1 from '@bravobit/bb-foundation/localize';

const createMask = (options) => typeof options === 'string' ? { mask: options } : options;

const InputmaskConstructor = _Inputmask.default || _Inputmask;
class BbInputMask {
    constructor(_ngZone, _platform, _renderer, _elementRef, _control) {
        this._ngZone = _ngZone;
        this._platform = _platform;
        this._renderer = _renderer;
        this._elementRef = _elementRef;
        this._control = _control;
        // State.
        this.inputMaskPlugin = null;
        this.nativeInputElement = null;
        this._inputMaskOptions = null;
        this._onChange = () => ({});
        this.onInput = (_) => ({});
        this.onTouched = () => ({});
        this.validate = (control) => !control.value || !this.inputMaskPlugin || this.inputMaskPlugin?.isValid()
            ? null
            : { inputMask: true };
        this.nativeInputElement = this._elementRef.nativeElement;
        if (this._control != null) {
            this._control.valueAccessor = this;
        }
    }
    set bbInputMask(mask) {
        if (!mask) {
            return;
        }
        this._inputMaskOptions = mask;
        this.updateMask();
    }
    ngOnInit() {
        const control = this._control.control;
        if (!control) {
            return;
        }
        control.setValidators(control.validator ? [control.validator, this.validate] : [this.validate]);
        control.updateValueAndValidity();
    }
    ngOnDestroy() {
        this.destroyMask();
    }
    writeValue(value) {
        if (!this.nativeInputElement) {
            return;
        }
        this._renderer.setProperty(this.nativeInputElement, 'value', value ?? '');
    }
    registerOnChange(fn) {
        this._onChange = fn;
        const parser = this._inputMaskOptions?.parser;
        this.onInput = value => this._onChange(parser && value ? parser(value) : value);
    }
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    setDisabledState(isDisabled) {
        if (!this.nativeInputElement) {
            return;
        }
        this._renderer.setProperty(this.nativeInputElement, 'disabled', isDisabled);
    }
    updateMask() {
        this.destroyMask();
        this.createMask();
        this.registerOnChange(this._onChange);
    }
    createMask() {
        const element = this._elementRef.nativeElement;
        if (!this._platform.isBrowser || !element || this._inputMaskOptions === null || Object.keys(this._inputMaskOptions).length === 0) {
            return;
        }
        const { parser, ...options } = this._inputMaskOptions;
        this.inputMaskPlugin = this._ngZone.runOutsideAngular(() => new InputmaskConstructor(options).mask(element));
        const control = this._control.control;
        if (!control) {
            return;
        }
        setTimeout(() => control.updateValueAndValidity());
    }
    destroyMask() {
        this.inputMaskPlugin?.remove();
        this.inputMaskPlugin = null;
    }
}
BbInputMask.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.0.4", ngImport: i0, type: BbInputMask, deps: [{ token: i0.NgZone }, { token: i1.Platform }, { token: i0.Renderer2 }, { token: i0.ElementRef }, { token: i2.NgControl, optional: true, self: true }], target: i0.ɵɵFactoryTarget.Directive });
BbInputMask.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "14.0.4", type: BbInputMask, selector: "input[bbInputMask]", inputs: { bbInputMask: "bbInputMask" }, host: { listeners: { "input": "onInput($event.target.value)", "blur": "onTouched($event.target.value)" } }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.0.4", ngImport: i0, type: BbInputMask, decorators: [{
            type: Directive,
            args: [{
                    selector: 'input[bbInputMask]'
                }]
        }], ctorParameters: function () { return [{ type: i0.NgZone }, { type: i1.Platform }, { type: i0.Renderer2 }, { type: i0.ElementRef }, { type: i2.NgControl, decorators: [{
                    type: Optional
                }, {
                    type: Self
                }] }]; }, propDecorators: { bbInputMask: [{
                type: Input
            }], onInput: [{
                type: HostListener,
                args: ['input', ['$event.target.value']]
            }], onTouched: [{
                type: HostListener,
                args: ['blur', ['$event.target.value']]
            }] } });

class Masking {
    constructor(_localize) {
        this._localize = _localize;
    }
    date(format = 2 /* DateMaskFormat.YearMonthDay */, separator = '-') {
        const dateParts = this.getDateParts(format);
        const inputFormat = dateParts.join(separator);
        const outputFormat = 'yyyy-MM-dd';
        const escape = /[.*+?^${}()\/|[\]\\]/g;
        const escapedSeparator = separator.replace(escape, '\\$&');
        const inputFormatRegexString = dateParts
            .map(item => `(?<${item}>[0-9]{${item?.length}})`)
            .join(escapedSeparator);
        const inputFormatRegex = new RegExp(`^${inputFormatRegexString}$`);
        return createMask({
            alias: 'datetime',
            inputFormat: inputFormat.toLowerCase(),
            outputFormat: outputFormat.toLowerCase(),
            nullable: false,
            inputmode: 'numeric',
            onBeforeMask: (initialValue) => {
                return initialValue ? formatDate(initialValue, inputFormat, 'en-US') : null;
            },
            onBeforePaste: (value) => {
                return value;
            },
            parser: (value) => {
                if (!value || typeof value !== 'string' || !inputFormatRegex.test(value)) {
                    return null;
                }
                const matches = value.match(inputFormatRegex);
                const year = +matches?.groups?.['yyyy'];
                const month = +matches?.groups?.['MM'] - 1;
                const day = +matches?.groups?.['dd'];
                const date = new Date(year, month, day);
                return date instanceof Date && !isNaN(date)
                    ? formatDate(date, outputFormat, 'en-US')
                    : null;
            }
        });
    }
    currency(code, options) {
        const locale = this._localize?.current?.locale ?? 'en';
        const symbol = options?.symbol ?? getCurrencySymbol(code, options?.symbolFormat ?? 'narrow', locale);
        const decimal = options?.decimalCharacter ?? getLocaleNumberSymbol(locale, NumberSymbol.CurrencyDecimal);
        const group = options?.groupCharacter ?? getLocaleNumberSymbol(locale, NumberSymbol.CurrencyGroup);
        const digits = options?.digits ?? getNumberOfCurrencyDigits(code);
        const spacer = options?.spacer ?? ' ';
        return createMask({
            alias: 'numeric',
            digits: digits,
            prefix: `${symbol}${spacer}`,
            radixPoint: decimal ?? '.',
            groupSeparator: group ?? ',',
            digitsOptional: options?.digitsOptional ?? false,
            autoUnmask: true,
            unmaskAsNumber: true,
            nullable: false,
            rightAlign: false,
            showMaskOnHover: false,
            showMaskOnFocus: false,
            inputmode: digits > 0 ? 'decimal' : 'numeric'
        });
    }
    getDateParts(format) {
        switch (format) {
            case 0 /* DateMaskFormat.DayMonthYear */:
                return ['dd', 'MM', 'yyyy'];
            case 1 /* DateMaskFormat.MonthDayYear */:
                return ['MM', 'dd', 'yyyy'];
            case 3 /* DateMaskFormat.YearDayMonth */:
                return ['yyyy', 'dd', 'MM'];
            case 2 /* DateMaskFormat.YearMonthDay */:
            default:
                return ['yyyy', 'MM', 'dd'];
        }
    }
}
Masking.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.0.4", ngImport: i0, type: Masking, deps: [{ token: i1$1.Localize, optional: true }], target: i0.ɵɵFactoryTarget.Injectable });
Masking.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "14.0.4", ngImport: i0, type: Masking, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.0.4", ngImport: i0, type: Masking, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return [{ type: i1$1.Localize, decorators: [{
                    type: Optional
                }] }]; } });

class MaskingModule {
}
MaskingModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.0.4", ngImport: i0, type: MaskingModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
MaskingModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "14.0.4", ngImport: i0, type: MaskingModule, declarations: [BbInputMask], exports: [BbInputMask] });
MaskingModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "14.0.4", ngImport: i0, type: MaskingModule });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.0.4", ngImport: i0, type: MaskingModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [BbInputMask],
                    exports: [BbInputMask]
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { BbInputMask, Masking, MaskingModule, createMask };
//# sourceMappingURL=bravobit-bb-foundation-masking.mjs.map
